import numpy as np
import shutil
import rappmysql
from rappmysql.mysqlquerys import connect
from rappmysql.mysqlquerys import mysql_rm
from datetime import datetime, timedelta, date
from dateutil.relativedelta import relativedelta
import traceback
import sys, os
import time
import pathlib
import zipfile
import csv

np.set_printoptions(linewidth=250)
compName = os.getenv('COMPUTERNAME')
# try:
#     compName = os.getenv('COMPUTERNAME')
#     if compName == 'DESKTOP-5HHINGF':
#         ini_users = r"D:\Python\MySQL\users.ini"
#         ini_chelt = r"D:\Python\MySQL\cheltuieli.ini"
#         ini_masina = r"D:\Python\MySQL\masina.ini"
#         report_dir = r"D:\Python\MySQL\onlineanywhere\static"
#     else:
#         ini_users = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         ini_chelt = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         ini_masina = r"C:\_Development\Diverse\pypi\cfgm.ini"
#         # ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"
#         report_dir = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\static"
# except:
#     ini_users = '/home/radum/mysite/static/wdb.ini'

app_masina_tables = {'all_cars': 'id_users', 'masina': 'id_users'}
app_masina_gui = ['masina.ui', 'add_auto.ui']
path2src = pathlib.Path(__file__)
car_list = path2src.resolve(path2src).parent / 'static' / 'car_list.csv'


class DB_Connection:
    def __init__(self, ini_masina):
        if isinstance(ini_masina, dict):
            self.credentials = ini_masina
        else:
            self.conf = connect.Config(ini_masina)
            self.credentials = self.conf.credentials
        self.auto_db = mysql_rm.DataBase(self.credentials)
        self.all_cars_table = mysql_rm.Table(self.credentials, 'all_cars')
        self.alimentari = mysql_rm.Table(self.credentials, 'masina')

    def close_connection(self):
        self.alimentari.db.close()
        self.all_cars_table.db.close()
        self.auto_db.db.close()

# class CheckAutoRequiredFiles:
#     def __init__(self, ini_file):
#         print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
#                                                                   sys._getframe().f_back.f_code.co_name))
#         if isinstance(ini_file, dict):
#             self.credentials = ini_file
#         else:
#             self.conf = connect.Config(ini_file)
#             self.credentials = self.conf.credentials
#         self.auto_db = None
#         path2GUI = pathlib.Path(__file__)
#         self.path2GUI = path2GUI.resolve(path2GUI).parent / 'GUI'
#         self.pth2SQLtables = os.path.join(os.path.dirname(__file__), 'static', 'sql')
#         self.check_requirements()
#
#     def check_requirements(self):
#         self.check_connection_to_database()
#         self.check_sql_gui_files_existance()
#         self.check_tabels_in_database()
#
#     def check_connection_to_database(self):
#         print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(
#             __name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
#         if not isinstance(self.credentials, dict):
#             raise RuntimeError('Credentials not dict')
#         self.auto_db = mysql_rm.DataBase(self.credentials)
#         if not self.auto_db.is_connected:
#             raise RuntimeError('Could not connect to database')
#         print('Connected to auto_db:', self.auto_db.is_connected)
#
#     def check_sql_gui_files_existance(self, ):
#         print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
#                                                                   sys._getframe().f_back.f_code.co_name))
#         for table in app_masina_tables.keys():
#             sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
#             print('##sql_table_template##', sql_file)
#             if not os.path.exists(sql_file):
#                 raise FileNotFoundError('{}'.format(sql_file))
#
#         for gui in app_masina_gui:
#             sql_file = os.path.join(self.path2GUI, gui)
#             print('##gui_file##', sql_file)
#             if not os.path.exists(sql_file):
#                 raise FileNotFoundError('{}'.format(sql_file))
#
#     def check_tabels_in_database(self):
#         print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
#                                                                   sys._getframe().f_back.f_code.co_name))
#         for table in app_masina_tables.keys():
#             # print('##table##', table)
#             sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
#             if table not in self.auto_db.allAvailableTablesInDatabase:
#                 print('Table {} not in database...creating it from:\n{}'.format(table, sql_file))
#                 self.auto_db.createTableFromFile(sql_file, table)
#             else:
#                 varName = 'table_{}'.format(table)
#                 loc = locals()
#                 # print(loc)
#                 exec("{} = mysql_rm.Table(self.credentials, '{}')".format(varName, table), globals(), loc)
#                 varName = loc[varName]
#                 same = varName.compare_sql_file_to_sql_table(sql_file)
#                 if same is not True:
#                     print(same)


class AutoApp:
    def __init__(self, auto_db, all_cars_table, alimentari, user_id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.auto_db = auto_db
        self.all_cars_table = all_cars_table
        self.alimentari = alimentari
        self.id = user_id

    @property
    def masini(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        masini = {}
        matches = ('id_users', self.id)
        cars_rows = self.all_cars_table.returnRowsWhere(matches)
        if cars_rows:
            for row in cars_rows:
                print('***row', row)
                indx_id = self.all_cars_table.columnsNames.index('id')
                indx_brand = self.all_cars_table.columnsNames.index('brand')
                indx_model = self.all_cars_table.columnsNames.index('model')
                table_name = '{}_{}'.format(row[indx_brand], row[indx_model])
                masini[row[indx_id]] = table_name
        return masini

    @property
    def electric_providers(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'electric')]
        col = self.alimentari.returnCellsWhere('eProvider', matches)
        electric_providers = list(set(col))
        # electric_providers = ['eCharge', 'MyHyundai', 'EnBW', 'SWM_Plus', 'SWM']
        return electric_providers

    def add_car(self, brand, model, car_type):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand = brand.lower()
        model = model.lower()
        car_type = car_type.lower()
        cols = ('id_users', 'brand', 'model', 'cartype')
        vals = (self.id, brand, model, car_type)
        matches = [('id_users', self.id), ('brand', brand), ('model', model), ('cartype', car_type)]
        existing_row = self.all_cars_table.returnRowsWhere(matches)
        # print('existing_row', existing_row)
        if existing_row:
            print('car already existing at id {}'.format(existing_row[0][0]))
            return
        else:
            self.all_cars_table.addNewRow(cols, vals)
            # new_auto_table = '{}_{}'.format(brand, model)
            # new_auto_table = new_auto_table.lower()
            # if new_auto_table in self.auto_db.allAvailableTablesInDatabase:
            #     print('table {} existing in database'.format(new_auto_table))
            # else:
            #     pth_auto_template = os.path.join(os.path.dirname(__file__), 'static', 'sql', 'auto_template.sql')
            #     self.auto_db.createTableFromFile(pth_auto_template, new_auto_table)

    def export_car_sql(self, car_id, export_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        all_cars_ident = {'id_users': self.id, 'id': car_id}
        masina_ident = {'id_users': self.id, 'id_all_cars': car_id}
        tables = {'all_cars': all_cars_ident,
                  'masina': masina_ident}

        tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
        output_dir = os.path.join(os.path.dirname(__file__), 'static', 'backup_profile',
                                  '{:09d}'.format(self.id),
                                  '{:09d}'.format(car_id),
                                  '{}_{:09d}'.format(tim, car_id))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        if export_files:
            sql_query = self.auto_db.return_sql_text(tables, export_files=output_dir)
        else:
            sql_query = self.auto_db.return_sql_text(tables)

        output_sql_file = os.path.join(output_dir, '{}_{:09d}.sql'.format(tim, self.id))
        FILE = open(output_sql_file, "w", encoding="utf-8")
        FILE.writelines(sql_query)
        FILE.close()
        #####
        output_zip = os.path.join(os.path.dirname(output_dir), '{}.zip'.format(output_dir))
        zip_file = self.zip_profile_files(output_dir, output_zip)
        if os.path.exists(zip_file):
            shutil.rmtree(output_dir)
        print('finished backup')

        return output_sql_file

    def import_car_with_files(self, zip_file, import_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        output_dir, file = os.path.split(zip_file)
        src_dir = self.unzip_profile_files(zip_file, output_dir)
        src_dir = os.path.join(src_dir, file[:-4])
        if not os.path.exists(src_dir):
            raise RuntimeError('Missing Folder {}'.format(src_dir))

        sql_files = [x for x in os.listdir(src_dir) if x.endswith('.sql')]
        sql_file = os.path.join(src_dir, sql_files[0])
        # print(sql_file)
        # return
        self.auto_db.run_sql_file(sql_file)
        if import_files:
            attachments = [x for x in os.listdir(src_dir) if
                           (x.endswith('.jpg') or
                            x.endswith('.pdf') or
                            x.endswith('.csv') or
                            x.endswith('.CSV')
                            )]
            tab = []
            for file_name in attachments:
                try:
                    # print(file_name)
                    table_id, orig_name = file_name.split('+')
                    fil = os.path.join(src_dir, file_name)
                    self.alimentari.changeCellContent('file', fil, 'id', table_id)
                    self.alimentari.changeCellContent('file_name', str(orig_name), 'id', table_id)
                    # print(id_users, table_name, table_id, orig_name)
                except:
                    print('could not import {}, name not ok'.format(file_name))
                tup = (table_id, orig_name, file_name)
                tab.append(tup)
        if os.path.exists(src_dir):
            shutil.rmtree(src_dir)

    def get_id_all_cars(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand, model = table_name.split('_')
        matches = [('id_users', self.id),
                   ('brand', brand),
                   ('model', model),
                   ]
        # print(matches)
        id_all_cars = self.all_cars_table.returnCellsWhere('id', matches)[0]
        return id_all_cars

    def delete_auto(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        brand, model = table_name.split('_')
        matches = [('id_users', self.id), ('brand', brand), ('model', model)]
        id_car = self.all_cars_table.returnCellsWhere('id', matches)
        row_id = int(id_car[0])
        condition = ('id', row_id)
        self.all_cars_table.delete_multiple_rows(condition)

    def erase_autoapp_traces(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # for i in self.chelt_db.checkProcess():
        #     print(i)
        self.auto_db.killAllProcess()
        self.auto_db.drop_table_list(list(app_masina_tables.keys()))

    def unzip_profile_files(self, src_file, output_dir):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        with zipfile.ZipFile(src_file, 'r') as zip_ref:
            zip_ref.extractall(output_dir)
        return output_dir

    def zip_profile_files(self, src_dir, output_file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        relroot = os.path.abspath(os.path.join(src_dir, os.pardir))
        with zipfile.ZipFile(output_file, "w", zipfile.ZIP_DEFLATED) as zip:
            for root, dirs, files in os.walk(src_dir):
                # add directory (needed for empty dirs)
                zip.write(root, os.path.relpath(root, relroot))
                for file in files:
                    filename = os.path.join(root, file)
                    if os.path.isfile(filename):  # regular files only
                        arcname = os.path.join(os.path.relpath(root, relroot), file)
                        zip.write(filename, arcname)
        return output_file

    def get_ppu_electric_provider(self, provider):
        matches = [#('id_users', '=', self.id),
                   ('eProvider', '=', provider),
                   ('type', '=', 'electric')
                   ]
        rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'DESC'))
        ppu = float(rows[0][self.alimentari.columnsNames.index('ppu')])
        # print('ppu', ppu)
        return ppu

    @property
    def all_cars_array(self, delimiter=','):
        array = []
        header = []
        with open(car_list, 'r', encoding='utf8', newline='') as file:
            reader = csv.reader(file, delimiter=delimiter)
            for i, row in enumerate(reader):
                if i < 1:
                    header.append(row)
                    continue
                array.append(row)
        array = np.atleast_2d(array)
        # print('#####array', array)
        array = array[array[:,0].argsort()]
        array = np.insert(array, 0, header, axis=0)

        return array


class Masina:
    def __init__(self, auto_db, all_cars_table, alimentari, id_users, id_car):
        '''
        :param ini_file:type=QFileDialog.getOpenFileName name=filename file_type=(*.ini;*.txt)
        '''
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.id_users = id_users
        self.id_car = id_car
        self.auto_db = auto_db
        self.all_cars_table = all_cars_table
        self.alimentari = alimentari

        self.types_of_costs = ["electric", "benzina", "intretinere", "asigurare", 'impozit', 'TüV', 'carwash', 'other']
        self.no_of_records = self.get_no_of_records()
        self.total_money = self.get_total_money()
        self.tot_benzina = self.get_tot_benzina()
        self.tot_electric = self.get_tot_electric()
        self.monthly_benzina = self.get_monthly_benzina()
        self.monthly_electric = self.get_monthly_electric()
        self.monthly = self.get_monthly()
        self.db_start_date = self.get_db_start_date()
        self.db_last_record_date = self.get_db_last_record_date()
        self.table_alimentari = self.get_table_alimentari()
        self.dict_totals = self.get_dict_totals()
        self.table_totals = self.get_table_totals()
        self.last_records = self.get_last_records()
        self.km_since_registered = self.get_km_since_registered()
        # self.refuelings_and_km = self.get_refuelings_and_km()

    # @property
    def get_no_of_records(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        return self.alimentari.noOfRows

    @property
    def default_interval(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        startDate = datetime(datetime.now().year - 1, datetime.now().month, datetime.now().day)
        endDate = datetime(datetime.now().year, datetime.now().month, datetime.now().day)
        return startDate, endDate

    @property
    def km_yearly(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        dates_days = list(self.km_since_registered.keys())
        days = dates_days[-1].days
        km = self.km_since_registered[dates_days[-1]]
        # print(days, km)
        yearly = (365*km) / days
        return round(yearly, 0)

    @property
    def km_monthly(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        dates_days = list(self.km_since_registered.keys())
        days = dates_days[-1].days
        km = self.km_since_registered[dates_days[-1]]
        # print(days, km)
        average_day_per_month = 365 / 12
        monthly = (average_day_per_month*km) / days
        return round(monthly, 0)

    @property
    def tank_capacity(self):
        matches = [('id_users', self.id_users), ('id', self.id_car)]
        tank_capacity = self.all_cars_table.returnCellsWhere('capa_tank', matches)
        return int(tank_capacity[0])

    def average_electric_consumption_since_last_refuel(self, actual_no_of_km, kwh_used):
        km_since_last_refuel = self.km_since_last_refuel(actual_no_of_km)
        electric_consumption_since_last_refuel = (kwh_used * 100) / km_since_last_refuel
        return round(electric_consumption_since_last_refuel, 1)

    # @property
    def get_total_money(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        col = self.alimentari.returnColumn('brutto')
        return round(sum(col), 2)

    # @property
    def get_tot_benzina(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'benzina'),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    # @property
    def get_tot_electric(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('type', 'electric'),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    def get_monthly(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            return round((self.monthly_benzina + self.monthly_electric), 2)
        except:
            return None

    # @property
    def get_km_since_registered(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            matches = [('id_users', '=', self.id_users),
                       ('id_all_cars', '=', self.id_car),
                       ('km', '!=', None)
                       ]
            rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('km', 'ASC'))
            first_registered_km_row = rows[0]
            last_registered_km_row = rows[-1]
            first_registered_km = first_registered_km_row[self.alimentari.columnsNames.index('km')]
            last_registered_km = last_registered_km_row[self.alimentari.columnsNames.index('km')]
            date_first_registered_km = first_registered_km_row[self.alimentari.columnsNames.index('data')]
            date_last_registered_km = last_registered_km_row[self.alimentari.columnsNames.index('data')]
            km_since_registered = last_registered_km - first_registered_km
            days_since_registered = date_last_registered_km - date_first_registered_km
            # return round(monthly, 2)
            # print(days_since_registered.days, type(days_since_registered))
            km_since_registered = {date_first_registered_km: first_registered_km,
                                   date_last_registered_km: last_registered_km,
                                   days_since_registered: km_since_registered}
            return km_since_registered
        except:
            return None
###############################################################################################

    def get_monthly_benzina(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            matches = [('type', 'benzina'),
                       ('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            rows = self.alimentari.returnRowsWhere(matches)
            rows = np.atleast_2d(rows)
            money = rows[:, self.alimentari.columnsNames.index('brutto')]
            start_date = min(rows[:, self.alimentari.columnsNames.index('data')])
            # finish_date = max(rows[:, self.alimentari.columnsNames.index('data')])
            total_money = round(sum(money), 2)
            # days = (finish_date - start_date).days
            days = (datetime.now().date() - start_date).days
            # print('start_date, finish_date, days, total_money', start_date, finish_date, days, total_money)
            # print('finish_date', type(finish_date))
            average_day_per_month = 365 / 12
            monthly = (average_day_per_month * total_money) / days
            return round(monthly, 2)
        except:
            return None

    # @property
    def get_monthly_electric(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            matches = [('type', 'electric'),
                       ('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            rows = self.alimentari.returnRowsWhere(matches)
            # print(rows)
            # print(type(rows))
            if rows:
                rows = np.atleast_2d(rows)
                money = rows[:, self.alimentari.columnsNames.index('brutto')]
                start_date = min(rows[:, self.alimentari.columnsNames.index('data')])
                # finish_date = max(rows[:, self.alimentari.columnsNames.index('data')])
                total_money = round(sum(money), 2)
                # days = (finish_date - start_date).days
                days = (datetime.now().date() - start_date).days
                # print('start_date, finish_date, days, total_money', start_date, days, total_money)

                average_day_per_month = 365 / 12
                monthly = (average_day_per_month * total_money) / days
                return round(monthly, 2)
        except:
            return None

    # @property
    def get_db_start_date(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        all_dates = self.alimentari.returnCellsWhere('data', matches)
        # print('**all_dates', all_dates, type(all_dates))
        if all_dates:
            start_date = min(all_dates)
        else:
            start_date = None
        return start_date

    # @property
    def get_db_last_record_date(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(
            __name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        try:
            # all_dates = self.alimentari.returnColumn('data')
            matches = [('id_users', self.id_users),
                       ('id_all_cars', self.id_car)]
            all_dates = self.alimentari.returnCellsWhere('data', matches)
            finish_date = max(all_dates)
            return finish_date
        except:
            return None

    # @property
    def get_table_alimentari(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        arr = [('', 'Alimentari[€]', 'Benzina[€]', 'Electric[€]')]
        if self.no_of_records > 0:
            total_alim = round(self.tot_benzina + self.tot_electric, 2)
            arr.append(('Monthly', self.monthly, self.monthly_benzina, self.monthly_electric))
            arr.append(('Total', total_alim, self.tot_benzina, self.tot_electric))
        else:
            arr.append(('Monthly', None, None, None))
            arr.append(('Total', None, None, None))

        arr = np.atleast_2d(arr)
        return arr

    def dict_last_months(self, delta_mths=12):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if not self.db_start_date:
            return None
        table_dict = {}
        try:
            for mnth in reversed(range(delta_mths+1)):
                dd = {}
                start_date = date.today() - relativedelta(months=+mnth)
                mnth_as_str = start_date.strftime("%B")
                startTime, endTime = self.get_monthly_interval(mnth_as_str, start_date.year)
                interval = '{}.{}'.format(mnth_as_str[:3], start_date.year)
                # print(startTime, endTime)
                tot = 0
                for t in self.types_of_costs:
                    matches = [('id_users', '=', self.id_users),
                               ('id_all_cars', '=', self.id_car),
                               ('type', '=', t),
                               ('data', '>=', startTime),
                               ('data', '<=', endTime)
                               ]
                    payments4Interval = self.alimentari.returnRowsQuery(matches)
                    if payments4Interval:
                        payments4Interval = np.atleast_2d(payments4Interval)
                        col = payments4Interval[:, self.alimentari.columnsNames.index('brutto')]
                        value = sum(col)
                        value = round(value, 2)
                        dd[t] = value
                        tot += value
                dd['total/row'] = round(tot, 2)
                table_dict[interval] = dd
            return table_dict
        except:
            return str(traceback.format_exc())

    # @property
    def get_dict_totals(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if not self.db_start_date:
            return None
        table_dict = {}
        try:
            for year in reversed(range(self.db_start_date.year, self.db_last_record_date.year + 1)):
                dd = {}
                startTime = datetime(year, 1, 1)
                endTime = datetime(year, 12, 31)
                tot = 0
                for t in self.types_of_costs:
                    # print('t', t)
                    matches = [('id_users', '=', self.id_users),
                               ('id_all_cars', '=', self.id_car),
                               ('type', '=', t),
                               ('data', '>=', startTime),
                               ('data', '<=', endTime)
                               ]
                    payments4Interval = self.alimentari.returnRowsQuery(matches)
                    if payments4Interval:
                        payments4Interval = np.atleast_2d(payments4Interval)
                        col = payments4Interval[:, self.alimentari.columnsNames.index('brutto')]
                        value = sum(col)
                        value = round(value, 2)
                        dd[t] = value
                        tot += value
                dd['total/row'] = round(tot, 2)
                table_dict[year] = dd
            return table_dict
        except:
            return str(traceback.format_exc())

    # @property
    def get_table_totals(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            table_head = self.types_of_costs.copy()
            table_totals = []
            for year, expenses in self.dict_totals.items():
                row = [year]
                total_per_year = 0
                for col in table_head:
                    if col in expenses.keys():
                        row.append(expenses[col])
                        total_per_year += expenses[col]
                    else:
                        row.append(0)
                row.append(round(total_per_year, 2))
                table_totals.append(tuple(row))
            table_head.insert(0, 'year')
            table_head.append('tot/year')
            table_totals.insert(0, tuple(table_head))
            table_totals = np.atleast_2d(table_totals)
            return table_totals
        except:
            return None

    # @property
    def get_last_records(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print(len(tuple(self.alimentari.columnsNames)))
        table_head = self.alimentari.columnsNames.copy()
        table_head.remove('file')
        last_records = [tuple(table_head)]
        for typ in self.types_of_costs:
            matches = [('id_users', self.id_users),
                       ('id_all_cars', self.id_car),
                       ('type', typ)]
            table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))
            if table:
                last_records.append(tuple(table[0]))
        last_records = np.atleast_2d(last_records)
        return last_records

    def delete_row(self, row_id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if isinstance(row_id, str):
            row_id = int(row_id)
        condition = ('id', row_id)
        self.alimentari.delete_multiple_rows(condition)

    def get_monthly_interval(self, month: str, year):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        mnth = datetime.strptime(month, "%B").month
        startDate = datetime(year, mnth, 1)

        if mnth != 12:
            lastDayOfMonth = datetime(year, mnth + 1, 1) - timedelta(days=1)
        else:
            lastDayOfMonth = datetime(year + 1, 1, 1) - timedelta(days=1)

        return startDate, lastDayOfMonth

    # def get_all_alimentari(self):
    #     print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
    #                                                               sys._getframe().f_back.f_code.co_name))
    #     cols = []
    #     for k, v in self.alimentari.columnsDetProperties.items():
    #         if v[0] == 'longblob':
    #             continue
    #         cols.append(k)
    #     alimentari = self.alimentari.returnColumns(cols)
    #     # alimentari = self.alimentari.returnAllRecordsFromTable()
    #     alimentari = np.atleast_2d(alimentari)
    #     alimentari = np.insert(alimentari, 0, cols, axis=0)
    #     return alimentari

    def get_alimentari_for_interval_type(self, selectedStartDate, selectedEndDate, alim_type):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        matches = [('data', (selectedStartDate, selectedEndDate)),
                   ('id_users', self.id_users),
                   ('id_all_cars', self.id_car)]
        if alim_type:
            matches.append(('type', alim_type))
        print(matches)
        table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))

        if table:
            table_head = []
            for col_name, prop in self.alimentari.columnsDetProperties.items():
                # print(col_name, prop)
                if prop[0] == 'longblob':
                    continue
                table_head.append(col_name)
            arr = np.atleast_2d(table)
            arr = np.insert(arr, 0, np.array(table_head), axis=0)
        else:
            arr = np.atleast_2d(np.array(self.alimentari.columnsNames))
        return arr

    def upload_file(self, file_name, id):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.alimentari.changeCellContent('file', file_name, 'id', id)
        pth, file_name = os.path.split(file_name)
        self.alimentari.changeCellContent('file_name', file_name, 'id', id)

    def insert_new_alim(self, current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges,
                        km, comment, file, provider):
        '''
        :param data:type=dateTime name=date
        :param alim_type:type=comboBox name=alim_type items=[electric,benzina,TüV,intretinere]
        :param brutto:type=text name=brutto
        :param amount:type=text name=amount
        :param refuel:type=text name=refuel
        :param other:type=text name=other
        :param recharges:type=text name=recharges
        :param km:type=text name=km
        :param comment:type=text name=comment
        :param file:type=QFileDialog.getOpenFileName name=file
        '''
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if file:
            _, file_name = os.path.split(file)
            cols = ['id_users', 'id_all_cars', 'data', 'type', 'brutto', 'amount', 'refuel', 'other', 'recharges',
                    'ppu', 'km', 'comment', 'file', 'file_name', 'eProvider']
        else:
            cols = ['id_users', 'id_all_cars', 'data', 'type', 'brutto', 'amount', 'refuel', 'other', 'recharges',
                    'ppu', 'km', 'comment', 'eProvider']
        try:
            if isinstance(brutto, str) and ',' in brutto:
                brutto = brutto.replace(',', '.')
            brutto = float(brutto)
        except:
            brutto = None
        try:
            if isinstance(amount, str) and ',' in amount:
                amount = amount.replace(',', '.')
            elif amount == '':
                amount = 1
            amount = float(amount)
        except:
            amount = None
        try:
            if isinstance(refuel, str) and ',' in refuel:
                refuel = refuel.replace(',', '.')
            refuel = float(refuel)
        except:
            refuel = None
        try:
            if isinstance(other, str) and ',' in other:
                other = other.replace(',', '.')
            other = float(other)
        except:
            other = None
        try:
            if isinstance(recharges, str) and ',' in recharges:
                recharges = recharges.replace(',', '.')
            recharges = float(recharges)
        except:
            recharges = None
        try:
            km = int(km)
        except:
            km = None

        ppu = round(float(brutto) / float(amount), 3)
        if file:
            vals = [current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges, ppu, km,
                    comment, file, file_name, provider]
        else:
            vals = [current_id_users, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges, ppu, km,
                    comment, provider]

        self.alimentari.addNewRow(cols, tuple(vals))

    def create_sql_table(self, table_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        masina_sql = os.path.join(os.path.dirname(__file__), 'static', 'sql',
                                  'auto_template.sql')

        self.auto_db.createTableFromFile(masina_sql, table_name)

    def last_records_for_plot(self, delta_mths=12):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        plot_lr = {}
        start_date = date.today() - relativedelta(months=+delta_mths)
        total_interval = date.today() - start_date
        # print('start_date', start_date)
        last_rec_tab_head = list(self.last_records[0])
        # print(last_rec_tab_head)
        for lr in self.last_records[1:]:
            # print(lr)
            ali_date = lr[last_rec_tab_head.index('data')]
            recharge_type = lr[last_rec_tab_head.index('type')]
            brutto = lr[last_rec_tab_head.index('brutto')]
            if ali_date > start_date:
                # print('BINGOOOOO')

                interval_since_lr = date.today() - ali_date
                # print(total_interval, interval_since_lr)
                # print(recharge_type, interval_since_lr/total_interval)
                label = '{}_{}_{}'.format(recharge_type, ali_date, brutto)
                # print(label, 1- interval_since_lr/total_interval)
                plot_lr[label] = 1 - interval_since_lr/total_interval
        return plot_lr

    @property
    def km_at_first_refuel(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'benzina'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='km', matches=matches)
        rows = [x for x in rows if x is not None]
        return min(rows)

    @property
    def date_at_first_refuel(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'benzina'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='data', matches=matches)
        return min(rows)

    @property
    def liters_tanked_at_first_refuel(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'benzina'),
                   ('data', self.date_at_first_refuel),
                   ]
        rows = self.alimentari.returnCellsWhere(col='amount', matches=matches)
        return float(rows[0])

    @property
    def km_at_last_refuel(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'benzina'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='km', matches=matches)
        rows = [x for x in rows if x is not None]
        return max(rows)

    @property
    def date_at_last_refuel(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'benzina'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='data', matches=matches)
        return max(rows)

    @property
    def km_driven_between_first_and_last_refueling(self):
        return self.km_at_last_refuel - self.km_at_first_refuel

    @property
    def liters_refueled_between_first_and_last_refueling(self):
        matches = [('id_users', '=', self.id_users),
                   ('id_all_cars', '=', self.id_car),
                   ('type', '=', 'benzina'),
                   ('data', '>', self.date_at_first_refuel)
                   ]
        rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))
        arr = np.atleast_2d(rows)
        liters_refueled_between_first_and_last_refueling = sum(arr[:, self.alimentari.columnsNames.index('amount')])
        return round(liters_refueled_between_first_and_last_refueling, 2)

    @property
    def liters_in_tank_when_i_bought_the_car(self):
        try:
            matches = [('id_users', self.id_users),
                       ('id_all_cars', self.id_car),
                       ('type', 'ini_benzina'),
                       ]
            liters_in_tank_when_i_bought_the_car = self.alimentari.returnCellsWhere(col='amount', matches=matches)
            return round(float(liters_in_tank_when_i_bought_the_car[0]), 2)
        except:
            return None

    @property
    def km_when_i_bought_the_car(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'ini_benzina'),
                   ]
        liters_in_tank_when_i_bought_the_car = self.alimentari.returnCellsWhere(col='km', matches=matches)
        return int(liters_in_tank_when_i_bought_the_car[0])

    @property
    def date_when_i_bought_the_car(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'ini_benzina'),
                   ]
        liters_in_tank_when_i_bought_the_car = self.alimentari.returnCellsWhere(col='data', matches=matches)
        return liters_in_tank_when_i_bought_the_car[0]

    @property
    def liters_used_until_last_refueling_without_last_refueling(self):
        try:
            matches = [('id_users', '=', self.id_users),
                       ('id_all_cars', '=', self.id_car),
                       ('type', '=', 'benzina'),
                       ('data', '<', self.date_at_last_refuel)
                       ]
            rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))
            # for i, row in enumerate(rows):
            #     print(i, row)
            #     brutto = row[self.alimentari.columnsNames.index('brutto')]
            #     comment = row[self.alimentari.columnsNames.index('comment')]
            #     km = row[self.alimentari.columnsNames.index('km')]
            #     amount = row[self.alimentari.columnsNames.index('amount')]
            #     # print(brutto, comment)
            #     # if i == 0 and brutto == 0 and comment == 'initial':
            #     #     print('BINGOOOOO')
            #     #     initial_km_stand = km
            #     #     liters_in_tank = amount
            arr = np.atleast_2d(rows)
            liters_refueled_between_first_and_last_refueling = sum(arr[:, self.alimentari.columnsNames.index('amount')])
            liters_in_tank_by_refueling = self.tank_capacity - self.liters_tanked_at_first_refuel
            # print('liters_in_tank_by_refueling', liters_in_tank_by_refueling)
            # print('***self.liters_refueled_between_first_and_last_refueling', self.liters_refueled_between_first_and_last_refueling)
            # print('***liters_refueled_between_first_and_last_refueling', liters_refueled_between_first_and_last_refueling)
            liters_used_till_first_refuel = self.liters_in_tank_when_i_bought_the_car - liters_in_tank_by_refueling
            # print('liters_used_till_first_refuel', liters_used_till_first_refuel)
            return round(liters_refueled_between_first_and_last_refueling + liters_used_till_first_refuel, 2)
        except:
            return None

    @property
    def liters_refueled_bis_datum(self):
        matches = [('id_users', '=', self.id_users),
                   ('id_all_cars', '=', self.id_car),
                   ('type', '=', 'benzina'),
                   ('data', '<=', datetime.now().date())
                   ]
        rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))
        # for i in rows:
        #     print(i)
        arr = np.atleast_2d(rows)
        liters_refueled_between_first_and_last_refueling = sum(arr[:, self.alimentari.columnsNames.index('amount')])
        return round(liters_refueled_between_first_and_last_refueling, 2)
### ELECTRIC ############################################################################

    @property
    def km_at_first_recharge(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'electric'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='km', matches=matches)
        rows = [x for x in rows if x is not None]
        return min(rows)

    @property
    def date_at_first_recharge(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'electric'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='data', matches=matches)
        return min(rows)

    @property
    def km_at_last_recharge(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'electric'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='km', matches=matches)
        rows = [x for x in rows if x is not None]
        return max(rows)

    @property
    def date_at_last_recharge(self):
        matches = [('id_users',  self.id_users),
                   ('id_all_cars',  self.id_car),
                   ('type', 'electric'),
                   ]
        rows = self.alimentari.returnCellsWhere(col='data', matches=matches)
        return max(rows)

    @property
    def km_driven_between_first_and_last_recharge(self):
        return self.km_at_last_recharge - self.km_at_first_recharge

    @property
    def kWh_recharged_between_first_and_last_recharge(self):
        try:
            matches = [('id_users', '=', self.id_users),
                       ('id_all_cars', '=', self.id_car),
                       ('type', '=', 'electric'),
                       ('data', '<', self.date_at_last_recharge)
                       ]
            rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))
            # for aaa in rows:
            #     print('++', aaa)
            arr = np.atleast_2d(rows)
            kWh_recharged_between_first_and_last_recharge = sum(arr[:, self.alimentari.columnsNames.index('amount')])
            return round(kWh_recharged_between_first_and_last_recharge, 2)
        except:
            return None

###############################################################################
    @property
    def long_term_consumption(self):
        try:
            average_electric_consumption = (100 * self.kWh_recharged_between_first_and_last_recharge) / self.km_driven_between_first_and_last_recharge
            if self.liters_in_tank_when_i_bought_the_car:
                average_fuel_consumption = (100 * self.liters_used_until_last_refueling_without_last_refueling) / self.km_at_last_refuel
                long_term_consumption = {'start': self.date_when_i_bought_the_car,
                                         'end': self.date_at_last_refuel,
                                         'km': self.km_at_last_refuel,
                                         'liters': self.liters_used_until_last_refueling_without_last_refueling,
                                         'average': round(average_fuel_consumption, 2),
                                         'date_at_first_recharge': self.date_at_first_recharge,
                                         'date_at_last_recharge': self.date_at_last_recharge,
                                         'km_driven_between_first_and_last_recharge': self.km_driven_between_first_and_last_recharge,
                                         'kWh_recharged_between_first_and_last_recharge': self.kWh_recharged_between_first_and_last_recharge,
                                         'average_electric_consumption': round(average_electric_consumption, 2),
                                         }
            else:
                average_fuel_consumption = (100 * self.liters_refueled_between_first_and_last_refueling) / self.km_driven_between_first_and_last_refueling
                long_term_consumption = {'start': self.date_at_first_refuel,
                                         'end': self.date_at_last_refuel,
                                         'km': self.km_driven_between_first_and_last_refueling,
                                         'liters': self.liters_refueled_between_first_and_last_refueling,
                                         'average': round(average_fuel_consumption, 2),
                                         'date_at_first_recharge': self.date_at_first_recharge,
                                         'date_at_last_recharge': self.date_at_last_recharge,
                                         'km_driven_between_first_and_last_recharge': self.km_driven_between_first_and_last_recharge,
                                         'kWh_recharged_between_first_and_last_recharge': self.kWh_recharged_between_first_and_last_recharge,
                                         'average_electric_consumption': round(average_electric_consumption, 2),
                                         }
            return long_term_consumption
        except:
            return None

    def calc_long_term_consumption(self, actual_no_of_km, benzin_left_in_tank):
        km_driven_bis_datum = actual_no_of_km - self.km_at_first_refuel
        liters_left_in_tank = self.tank_capacity * (benzin_left_in_tank / 100)
        # print('liters_left_in_tank', liters_left_in_tank)
        if self.liters_in_tank_when_i_bought_the_car:
            liters_used = self.liters_used_until_last_refueling_without_last_refueling + (self.tank_capacity - liters_left_in_tank)
            average_fuel_consumption = (100 * liters_used) / km_driven_bis_datum
            long_term_consumption = {'date_when_i_bought_the_car': self.date_when_i_bought_the_car,
                                     'today': datetime.now().date(),
                                     'km_driven_bis_datum': km_driven_bis_datum,
                                     'liters_used': liters_used,
                                     'average_fuel_consumption': round(average_fuel_consumption, 2),
                                     }
            return long_term_consumption
        else:
            liters_used = self.liters_refueled_bis_datum - liters_left_in_tank
            # print('liters_used', liters_used)
            average_fuel_consumption = (100 * liters_used) / km_driven_bis_datum
            long_term_consumption = {'date_at_first_refuel': self.date_at_first_refuel,
                                     'date now': datetime.now().date(),
                                     'km_driven_bis_datum': km_driven_bis_datum,
                                     'liters_used_bis_datum': round(liters_used, 2),
                                     'average_fuel_consumption': round(average_fuel_consumption, 2),
                                     }
            return long_term_consumption

    # @property
    def since_refueling_consumption(self, actual_no_of_km, benzin_left_in_tank, average_electric_consumption_since_last_refuel):
        liters_left_in_tank = self.tank_capacity * (benzin_left_in_tank / 100)
        liters_used = self.tank_capacity - liters_left_in_tank
        km_since_last_refuel = actual_no_of_km - self.km_at_last_refuel
        fuel_consumption_since_last_refuel = (liters_used * 100) / km_since_last_refuel
        electric_consumption_since_last_refuel = (average_electric_consumption_since_last_refuel * km_since_last_refuel) / 100
        long_term_consumption = {'date_at_last_refuel': self.date_at_last_refuel,
                                 'date_now': datetime.now().date(),
                                 'km_since_last_refuel': km_since_last_refuel,
                                 'liters_used': liters_used,
                                 'fuel_consumption_since_last_refuel': round(fuel_consumption_since_last_refuel, 2),
                                 'electric_consumption_since_last_refuel': round(electric_consumption_since_last_refuel, 2),
                                 }
        return long_term_consumption

    @property
    def vw_ab_tanken(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        try:
            table = [('alim_date', 'km_driven', 'used_fuel', 'average_fuel', 'km', 'days')]
            matches = [('id_users', '=', self.id_users),
                       ('id_all_cars', '=', self.id_car),
                       ('km', 'IS NOT', None),
                       ('type', '=', 'benzina')
                       ]
            rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))

            if self.liters_in_tank_when_i_bought_the_car:
                for i, row in enumerate(rows):
                    # print(i, row)
                    qty_alim = row[self.alimentari.columnsNames.index('amount')]
                    if i == 0:
                        end_date = row[self.alimentari.columnsNames.index('data')]
                        km = row[self.alimentari.columnsNames.index('km')]
                        liters_in_tank_by_refueling = self.tank_capacity - self.liters_tanked_at_first_refuel
                        qty_alim = round(self.liters_in_tank_when_i_bought_the_car - liters_in_tank_by_refueling, 2)
                        # print('liters_in_tank_by_refueling', liters_in_tank_by_refueling)
                        # print('self.liters_in_tank_when_i_bought_the_car', self.liters_in_tank_when_i_bought_the_car)
                        # print('qty_alim', qty_alim)
                        driven_km_between_refueling = km - self.km_when_i_bought_the_car
                        medium_fuel_consumption = round(100 * qty_alim / driven_km_between_refueling, 2)
                        driven_days_between_refueling = end_date - self.date_when_i_bought_the_car
                        tup = (end_date, driven_km_between_refueling, qty_alim, medium_fuel_consumption, km, driven_days_between_refueling.days)
                        table.append(tup)
                        continue
                    driven_days_between_refueling = row[self.alimentari.columnsNames.index('data')] - end_date
                    driven_km_between_refueling = row[self.alimentari.columnsNames.index('km')] - km
                    medium_fuel_consumption = round(100 * qty_alim / driven_km_between_refueling, 2)
                    km = row[self.alimentari.columnsNames.index('km')]
                    end_date = row[self.alimentari.columnsNames.index('data')]
                    tup = (end_date, driven_km_between_refueling, qty_alim, medium_fuel_consumption, km,
                           driven_days_between_refueling.days)
                    table.append(tup)

            else:
                km = None
                data = None
                driven_km_between_refueling = None
                driven_days_between_refueling = None
                medium_fuel_consumption = None
                start_date = None
                end_date = None
                for i, row in enumerate(rows):
                    # print(i, row)
                    if km is not None:
                        driven_km_between_refueling = row[self.alimentari.columnsNames.index('km')] - km
                        start_date = data
                        end_date = row[self.alimentari.columnsNames.index('data')]
                        driven_days_between_refueling = row[self.alimentari.columnsNames.index('data')] - data
                    km = row[self.alimentari.columnsNames.index('km')]
                    data = row[self.alimentari.columnsNames.index('data')]
                    qty_alim = row[self.alimentari.columnsNames.index('amount')]
                    if driven_km_between_refueling is not None:
                        medium_fuel_consumption = round(100 * qty_alim / driven_km_between_refueling, 2)
                    # print('km', km, )
                    # print('driven_km_between_refueling', start_date, end_date, driven_km_between_refueling, qty_alim, medium_fuel_consumption, driven_days_between_refueling)
                    tup = (km, end_date, qty_alim, driven_km_between_refueling, medium_fuel_consumption,
                           driven_days_between_refueling)
                    table.append(tup)
            arr = np.atleast_2d(table)
            return arr
        except:
            print(traceback.format_exc())
            return None

    # def get_refuelings_and_km(self):
    #     print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
    #                                                               sys._getframe().f_back.f_code.co_name))
    #     try:
    #         table = [('km', 'alim_date', 'qty_alim', 'driven_km_between_refueling', 'medium_fuel_consumption', 'driven_days_between_refueling')]
    #         matches = [('id_users', '=', self.id_users),
    #                    ('id_all_cars', '=', self.id_car),
    #                    ('km', '!=', None),
    #                    ('type', '=', 'benzina')
    #                    ]
    #         rows = self.alimentari.returnRowsQuery(matches=matches, order_by=('data', 'ASC'))
    #         km = None
    #         data = None
    #         driven_km_between_refueling = None
    #         driven_days_between_refueling = None
    #         medium_fuel_consumption = None
    #         start_date = None
    #         end_date = None
    #         for i, row in enumerate(rows):
    #             # print(i, row)
    #             if km is not None:
    #                 driven_km_between_refueling = row[self.alimentari.columnsNames.index('km')] - km
    #                 start_date = data
    #                 end_date = row[self.alimentari.columnsNames.index('data')]
    #                 driven_days_between_refueling = row[self.alimentari.columnsNames.index('data')] - data
    #             km = row[self.alimentari.columnsNames.index('km')]
    #             data = row[self.alimentari.columnsNames.index('data')]
    #             qty_alim = row[self.alimentari.columnsNames.index('amount')]
    #             if driven_km_between_refueling is not None:
    #                 medium_fuel_consumption = round(100 * qty_alim / driven_km_between_refueling, 2)
    #             # print('km', km, )
    #             # print('driven_km_between_refueling', start_date, end_date, driven_km_between_refueling, qty_alim, medium_fuel_consumption, driven_days_between_refueling)
    #             tup = (km, end_date, qty_alim, driven_km_between_refueling, medium_fuel_consumption, driven_days_between_refueling)
    #             table.append(tup)
    #         arr = np.atleast_2d(table)
    #         return arr
    #     except:
    #         print(traceback.format_exc())
    #         return None


def main():
    script_start_time = time.time()
    selectedStartDate = datetime(2024, 7, 15, 0, 0, 0)
    selectedEndDate = datetime(2025, 7, 15, 0, 0, 0)

    auto_db_connection = DB_Connection(rappmysql.ini_masina)

    # auto_app = AutoApp(auto_db=auto_db_connection.auto_db,
    #                    all_cars_table=auto_db_connection.all_cars_table,
    #                    alimentari=auto_db_connection.alimentari,
    #                    user_id=1)
    # print(auto_app.all_cars_array)
    # return
    app_masina = Masina(auto_db=auto_db_connection.auto_db,
                        all_cars_table=auto_db_connection.all_cars_table,
                        alimentari=auto_db_connection.alimentari,
                        id_users=5,
                        id_car=15)
    # alimentari = app_masina.get_alimentari_for_interval_type(selectedStartDate, selectedEndDate, None)
    # for i in alimentari:
    #     print(i)
    # print(len(alimentari))
    # print('app_masina.date_at_first_refuel', app_masina.date_at_first_refuel)
    # print('app_masina.km_at_first_refuel', app_masina.km_at_first_refuel)
    # print('app_masina.date_at_last_refuel', app_masina.date_at_last_refuel)
    # print('app_masina.km_at_first_refuel', app_masina.km_at_last_refuel)
    # print('app_masina.km_driven_between_first_and_last_refueling', app_masina.km_driven_between_first_and_last_refueling)
    # print('app_masina.kWh_recharged_between_first_and_last_recharge', app_masina.kWh_recharged_between_first_and_last_recharge)
    # print('app_masina.kWh_recharged_between_first_and_last_recharge', app_masina.km_at_last_recharge)
    print('app_masina.liters_tanked_at_first_refuel', app_masina.liters_tanked_at_first_refuel)
    print('app_masina.liters_in_tank_when_i_bought_the_car', app_masina.liters_in_tank_when_i_bought_the_car)
    print('app_masina.liters_used_until_last_refueling', app_masina.liters_used_until_last_refueling_without_last_refueling)
    print('app_masina.liters_used_until_last_refueling', app_masina.km_at_last_refuel)
    # print('app_masina.liters_used_until_last_refueling', app_masina.liters_used_until_last_refueling_without_last_refueling)

    print('app_masina.get_refuelings_and_km', app_masina.vw_ab_tanken)
    print()
    return
    print(50 * ' ', '##vw_long_term_consumption##')
    for k, v in app_masina.long_term_consumption.items():
        print(k, v)
    print(50 * ' ', '##calc_vw_long_term_consumption##')
    calc_vw_long_term_consumption =app_masina.calc_long_term_consumption(2570, 81)
    for k, v in calc_vw_long_term_consumption.items():
        print(k, v)

    return
    print(50 * ' ', '##since_refueling_consumption##')
    consumption_since_refueling = app_masina.since_refueling_consumption(2073, 36, 13.9)
    for k, v in consumption_since_refueling.items():
        print(k, v)

    print(50 * ' ', '##bis_datum_consumption##')
    bis_datum_consumption = app_masina.bis_datum_consumption(2073, 36, 13.9)
    for k, v in bis_datum_consumption.items():
        print(k, v)

    return
    # print(app_masina.tot_benzina)
    # print(app_masina.tot_electric)
    # print(app_masina.monthly_benzina)
    # print(app_masina.monthly_electric)
    # print(app_masina.km_yearly)
    # print(app_masina.km_monthly)
    print(app_masina.get_refuelings_and_km())
    # print(app_masina.last_refuel_date)
    print(app_masina.km_at_last_refuel_date)
    # print(app_masina.days_since_last_refuel)
    # print(app_masina.km_since_last_refuel(1954))
    # print(app_masina.tank_capacity)
    print('****', )
    print(app_masina.average_electric_consumption_since_last_refuel(2073, 193.35))
    print(app_masina.electric_consumption_since_last_refuel(2073, 11))
    # print(app_masina.last_refuel_date)

    print(app_masina.calculate_average_consumption_from_first_to_last_refuel())


    # print(app_masina.total_amount_of_kwh_at_last_refuel_km())

    # print(app_masina.dict_totals)
    # print(app_masina.bkp_table_totals)
    # print(app_masina.monthly)
    # dict_last_months = app_masina.dict_last_months()
    # for interval, values in dict_last_months.items():
    #     print(interval)
    #     print(values)
        # print(type(values))
    # plot_lr = app_masina.last_records_for_plot()
    # print(plot_lr)
    # print(app_masina.table_totals)
    # print('app_masina.last_records')
    # print(app_masina.last_records)

    scrip_end_time = time.time()
    duration = scrip_end_time - script_start_time
    duration = time.strftime("%H:%M:%S", time.gmtime(duration))
    print('run time: {}'.format(duration))


if __name__ == '__main__':
    main()
# else:
#     sys.stdout = open(os.devnull, 'w')


import os.path
import mysql.connector as mysql
import re
import codecs
import sys
import traceback
import numpy as np
import subprocess
from datetime import datetime
from mysqlquerys import connect


class DataBase:
    def __init__(self, credentials):
        self.db = mysql.connect(**credentials)
        # self.db.set_session(autocommit=True)
        self.cursor = self.db.cursor()
        self.db_name = credentials['database']

    @property
    def is_connected(self):
        return self.db.is_connected()

    @property
    def dataBaseVersion(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.cursor.execute('SELECT version()')
        db_version = self.cursor.fetchone()
        return db_version

    @property
    def allAvailableTablesInDatabase(self):
        """ get all tables in schema"""
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        command = "SHOW TABLES"
        cur.execute(command)
        rows = cur.fetchall()
        tables = []
        for row in rows:
            tabName = row[0]
            tables.append(tabName)
        cur.close()

        return sorted(tables)

    def checkProcess(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = "SHOW PROCESSLIST"
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        return records

    def killProcess(self, processes):
        for process in processes:
            query = "KILL {}".format(process)
            self.cursor.execute(query)

    def createTableFromFile(self, file, newTableName):
        '''
        :param file: QFileDialog.getOpenFileName
        :return:
        '''
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        try:
            error = None
            command = ''
            f = open(file)
            for line in f.read().splitlines():
                command += line
                if ';' in line:
                    if re.search("^CREATE TABLE", command) or \
                            re.search("^ALTER TABLE", command) or \
                            re.search("^INSERT INTO", command):
                        match = re.findall(r"\`(.+?)\`", command)
                        tableName = match[0]
                        # newTableName = '{}'.format(tableName)
                        command = command.replace(tableName, newTableName)
                        # if 'REFERENCES' in command:
                        #     text = command.split('REFERENCES')[1]
                        #     match = re.findall(r"\`(.+?)\`", text)
                        #     referenceTable = match[0]
                        #     newRefName = "{} {}".format(self.schemaName, referenceTable)
                        #     refTab = '`{}`'.format(referenceTable)
                        #     newRef = '`{}`'.format(newRefName)
                        #     command = command.replace(refTab, newRef)
                    cur = self.db.cursor()
                    cur.execute(command)
                    command = ''
            f.close()
        except mysql.Error as err:
            print('ERROR mysql.Error: ', err.msg)
            error = (err.msg, command)
            cur.close()
        except Exception:
            print('ERR: ', traceback.format_exc())
            error = traceback.format_exc()
            cur.close()

        return error

    def createTableList(self, fileList):
        '''
        :param fileList: QFileDialog.getOpenFileNames
        :return: Error if exists, else None
        '''
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        errList = []
        while fileList:
            for i, file in enumerate(fileList):
                error = self.createTableFromFile(file)
                if error:
                    if error[0] == 'Cannot add foreign key constraint' \
                            or re.search("^relation.*does not exist$", error[0]):
                        errList.append(error[1])
                        fileList.pop(i)
                        continue
                    elif 'already exists' in error[0]:
                        fileList.pop(i)
                    else:
                        print('ERR createTablesFromFiles: ', error)
                        fileList.pop(i)
                else:
                    fileList.pop(i)

        probList = []
        for com in errList:
            try:
                print('Retrying... {}'.format(com))
                cur = self.db.cursor()
                cur.execute(com)
                cur.close()
            except Exception:
                print('ERR: ', traceback.format_exc())
                error = (traceback.format_exc(), com)
                probList.append(error)

        if probList:
            print('remaining errs:')
            for i in errList:
                print(i)
        else:
            print('Successfully')

    def drop_table(self, tableName):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = "DROP TABLE IF EXISTS {} CASCADE;".format(tableName)
        cur = self.db.cursor()
        cur.execute(query)
        cur.close()

    def drop_table_list(self, tableList):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        results = []
        while tableList:
            for i, tab in enumerate(tablelist):
                if self.connectionType == 'mysql':
                    exists = self.checkIfTableExists(tab)
                    if not exists:
                        print('table {} does not exist in database {}'.format(tab, self.dataBaseName))
                        tableList.pop(i)
                        continue
                    query = "SELECT table_name FROM information_schema.KEY COLUMN USAGE " \
                            "WHERE table_schema = %s AND referenced table_name = %s"
                    cur = self.db.cursor()
                    cur.execute(query, (self.dataBaseName, tab))
                    children = []
                    for cursor in cur.fetchall():
                        if cursor[0]:
                            children.append(cursor)
                    if not children:
                        res = self.drop_table(tab)
                        results.append(res)
                        tableList.pop(i)
                    else:
                        # if children make sure all of them are included in list
                        for child in children:
                            if child[0] not in tablelist:
                                print('if child not in tablelist:', tab, child, children)
                                err = self.drop_table(tab)
                                results.append(err)
                                tableList.pop(i)
                                break
                    cur.close()
                elif self.connectionType == 'postgresql':
                    res = self.drop_table(tab)
                    results.append(res)
                    tableList.pop(i)
        return results

    def rename_table(self, tableName, newName):
        # #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = ('RENAME TABLE {} TO {}'.format(tableName, newName))
        cur = self.db.cursor()
        try:
            cur.execute(query)
            cur.close
        except mysql.connector.Error as err:
            print(err.msg)

    def deleteAllDataInTable(self, tableList):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        for tab in tableList:
            query = ('DELETE FROM {}'.format(tab))
            cur.execute(query)
            self.db.commit()
        cur.close()

    def export_database(self, output_file):
        cmd = ['mysqldump', '-u', 'root', '-p', 'cheltuieli_desktop', '>', r"D:\Python\sql_tables\bbb.sql"]
        print('CMD', cmd)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
        out, err = p.communicate()
        if p.returncode != 0:
            print('returncode: ', p.returncode)
            print('Error: ', err)
        else:
            print('Done: {}'.format(cmd))

    def show_create_table(self, tableName):
        query = 'SHOW CREATE TABLE {}'.format(tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        records = records[0]

        cursor.close()
        values = []
        # for i in records[1]:
        #     print(i)
        #     # values.append(i[0])
        # # return values
        # filename = str(os.getenv("HOME")) + "/backup_" + now.strftime("%Y-%m-%d_%H:%M") + ".sql"
        filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\aa.sql"
        data = records[1]
        FILE = open(filename, "w")
        FILE.writelines(data)
        FILE.close()

    # def show_create_table_2(self, tableName):
    #     print('################')
    #     query = 'SELECT * FROM {}'.format(tableName)
    #     cursor = self.db.cursor()
    #     cursor.execute(query)
    #     records = cursor.fetchall()
    #     # print('++', query)
    #     data = ""
    #     for row in records:
    #         # print('ÄÄ', row)
    #         data += "INSERT INTO `" + str(tableName) + "` VALUES("
    #         first = True
    #         for field in row:
    #             if not first:
    #                 data += ', '
    #             data += '"' + str(field) + '"'
    #             first = False
    #
    #         data += ");\n"
    #     data += "\n\n"
    #
    #     values = []
    #     # for i in data:
    #     #     print(i)
    #     cursor.close()
    #     #     # values.append(i[0])
    #     # # return values
    #     # filename = str(os.getenv("HOME")) + "/backup_" + now.strftime("%Y-%m-%d_%H:%M") + ".sql"
    #     filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\bb.sql"
    #     # data = records[1]
    #     FILE = open(filename, "w")
    #     FILE.writelines(data)
    #     FILE.close()

    def export_tables_to_sql(self, tables, output_sql_file):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            cur.execute("SHOW CREATE TABLE `" + str(table) + "`;")
            data += "\n" + str(cur.fetchone()[1]) + ";\n\n"
            cur.execute("SELECT * FROM `" + str(table) + "`;")
            for row in cur.fetchall():
                data += "INSERT INTO `" + str(table) + "` VALUES("
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    data += '"' + str(field) + '"'
                    first = False

                data += ");\n"
            data += "\n\n"
        FILE = open(output_sql_file, "w")
        FILE.writelines(data)
        FILE.close()

    def backup_profile_with_id(self, tables, output_sql_file, user_id):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            # print('::table', table)
            cur.execute("SHOW CREATE TABLE `" + str(table) + "`;")
            data += "\n" + str(cur.fetchone()[1]) + ";\n\n"

            cur.execute("SELECT * FROM `" + str(table) + "` WHERE {}={};".format(tables[table], user_id))
            for row in cur.fetchall():
                data += "INSERT INTO `" + str(table) + "` VALUES("
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    data += '"' + str(field) + '"'
                    first = False

                data += ");\n"
            data += "\n\n"
        FILE = open(output_sql_file, "w")
        FILE.writelines(data)
        FILE.close()

    def backup_profile_without_id(self, tables, output_sql_file, user_id):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            print('::table', table)
            cur.execute("SHOW CREATE TABLE `" + str(table) + "`;")
            data += "\n" + str(cur.fetchone()[1]) + ";\n\n"

            cur.execute("SELECT * FROM `" + str(table) + "` where {}={};".format(tables[table], user_id))
            for row in cur.fetchall():
                data += "INSERT INTO `" + str(table) + "` VALUES("
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    data += '"' + str(field) + '"'
                    first = False

                data += ");\n"
            data += "\n\n"

        aa = data.split('\n')
        print(200*'+')
        for ii in aa:
            print(ii)
        print(200 * '+')
        # FILE = open(output_sql_file, "w")
        # FILE.writelines(data)
        # FILE.close()


class Table(DataBase):
    def __init__(self, credentials, tableName):
        super().__init__(credentials)
        self.tableName = tableName

    @property
    def noOfRows(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = 'SELECT COUNT(*) FROM {}'.format(self.tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        noOfRows = cursor.fetchone()[0]
        # rowNo = cursor.rowcount
        cursor.close()
        return noOfRows

    def lastRowId(self):
        query = ('SELECT id FROM {} ORDER BY id DESC LIMIT 1'.format(self.tableName))
        cur = self.db.cursor()
        cur.execute(query)
        lastId = cur.fetchonel()
        if lastId is None:
            return 0
        return lastId[0]

    @property
    def columnsNames(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cursor = self.db.cursor()
        query = 'DESC {}'.format(self.tableName)
        cursor.execute(query)
        res = cursor.fetchall()
        cols = []
        for col in res:
            cols.append(col[0])
        cursor.close()
        return cols

    @property
    def columnsDetProperties(self):
        query = 'DESC {}'.format(self.tableName)
        colNames = ['Field', 'Type', 'Null', 'Key', 'Default', 'Extra']
        cursor = self.db.cursor()
        cursor.execute(query)
        res = cursor.fetchall()
        cols = {}
        for col in res:
            colName, colType, null, key, default, extra = col
            if isinstance(colType, bytes):
                colType = str(colType.decode("utf-8"))
            cols[colName] = [colType, null, key, default, extra]
        cursor.close()
        return cols

    @property
    def columnsProperties(self):
        #print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        cursor = self.db.cursor()
        query = ("SELECT table_name, column_name, data_type FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = '{}'").format(self.tableName)
        cursor.execute(query)
        res = cursor.fetchall()
        cols = {}
        for col in res:
            table_name, col_name, data_type = col
            cols[col_name] = data_type
        cursor.close()
        return cols

    @property
    def children_tables(self):
        query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE " \
                "WHERE table_schema = %s AND referenced_table_name = %s "
        # print(query)
        # print(self.db_name, self.tableName)
        cur = self.db.cursor()
        cur.execute(query, (self.db_name, self.tableName))
        children = []
        for cursor in cur.fetchall():
            print(cursor)
            if cursor[0]:
                children.append(cursor[0])
        return children

    def deleteRow(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = %s '.format(self.tableName, colName)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def delete_multiple_rows(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = {} '.format(self.tableName, colName, value)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def delete_multiple_rows_multiple_conditions(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = {} '.format(self.tableName, colName, value)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def convertToBinaryData(self, filename):
        # Convert digital data to binary format
        with open(filename, 'rb') as file:
            binaryData = file.read()
        return binaryData

    def addNewRow(self, columns, values):
        # print(len(columns), len(values))
        strCols = (('{}, ' * len(columns)).format(*columns))
        strCols = '({})'.format(strCols[:-2])
        strVals = ('%s,'*len(columns))
        strVals = '({})'.format(strVals[:-1])

        query = "INSERT INTO {} {} VALUES {}".format(self.tableName, strCols, strVals)
        #######
        print(query)
        for i in range(len(columns)):
            print(columns[i], values[i])
        #######
        if isinstance(values, int):
            values = (values, )
        elif isinstance(values, str):
            values = (values,)
        elif isinstance(values, tuple):
            print('values', values)
            new_vals = []
            for v in values:
                if isinstance(v, str):
                    if os.path.isfile(v):
                        v = self.convertToBinaryData(v)
                new_vals.append(v)
            values = tuple(new_vals)

        cursor = self.db.cursor()
        cursor.execute(query, values)
        self.db.commit()
        cursor.close()

        return cursor.lastrowid

    def insertColumns(self, column_name, column_definition, afterCol):
        if afterCol == 'FIRST':
            query = 'ALTER TABLE {} ADD COLUMN {} {} FIRST'.format(self.tableName, column_name, column_definition)
        else:
            query = 'ALTER TABLE {} ADD COLUMN {} {} AFTER {}'.format(self.tableName, column_name, column_definition, afterCol)

        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def returnAllRecordsFromTable(self):
        cur = self.db.cursor()
        query = ('SELECT * FROM {}'.format(self.tableName))
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnAllRecordsFromTableExceptBlob(self):
        cur = self.db.cursor()
        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {}'.format(self.tableName)
        print(query)
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnLastRecords(self, column, noOfRows2Return):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        query = ('SELECT * FROM {} ORDER BY {} DESC LIMIT %s'.format(self.tableName, column))
        cur.execute(query, (noOfRows2Return,))
        rows = cur.fetchall()
        cur.close()
        return rows

    def filterRows(self, matches, order_by=None):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        filterText = ''
        for match in matches:
            search_col, search_key = match
            if isinstance(search_key, tuple):
                min, max = search_key
                new = "{} > '{}' AND {} < '{}' AND ".format (search_col, min, search_col, max)
                filterText += new
            elif isinstance(search_key, list):
                new = "{} in {} AND ".format(search_col, tuple(search_key))
                filterText += new
            elif search_key == 'None' or search_key is None:
                new = "{} IS NULL AND ".format(search_col, search_key)
                filterText += new
            else:
                new = "{} = '{}' AND ".format(search_col, search_key)
                filterText += new

        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])

        # print(query)
        # query = "SELECT * FROM {} WHERE ".format(self.tableName) + filterText[:-4]
        if order_by:
            col, order = order_by
            txt = 'ORDER BY {} {}'.format(col, order)
            query += txt
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        return records

    def returnRowsWhere(self, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT * FROM {} WHERE {} = '{}'".format(self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT * FROM {} WHERE {} IN '{}'".format(self.tableName, searchCol, searchKey)
            if searchKey is None:
                query = "SELECT * FROM {} WHERE {} IS NULL".format(self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    new = '{} IS NULL AND '.format(searchCol)
                else:
                    new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT * FROM {} WHERE ".format(self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def returnRowsLike(self, column, keyWord):
        query = "SELECT * FROM {} WHERE {} LIKE '%{}%'".format(self.tableName,
                                                               column, keyWord)
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnRowsYoungerThan(self, column, timeP):
        query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, column, column)
        cur = self.db.cursor()
        cur.execute(query, (timeP, ))
        records = cur.fetchall()
        return records

    def returnRowsInInterval(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} > %s OR {} < %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def returnRowsOfYear(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} > %s AND {} < %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def returnRowsOutsideInterval(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} < %s AND {} > %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def get_column_type(self, column):
        colProps = self.columnsProperties[column]
        # print('µµµµµµµµµ', colProps)
        colType = colProps[0]
        return colType

    def modify2AutoIncrement(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {} AUTO_INCREMENT;'.format(self.tableName, column, colType)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def modifyType(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {};'.format(self.tableName, column, colType)
        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def changeCellContent(self, column2Modify, val2Moify, refColumn, refValue):
        try:
            query = "UPDATE {} SET {} = %s WHERE {} = %s".format(self.tableName, column2Modify, refColumn)
            print(query)
            cursor = self.db.cursor()
            if isinstance(val2Moify, str):
                if os.path.isfile(val2Moify):
                    # print('aaaaa', val2Moify, type(val2Moify))
                    val2Moify = self.convertToBinaryData(val2Moify)
                    # print('aaaaa', val2Moify, type(val2Moify))
            vals = (val2Moify, refValue)
            # print(vals)
            cursor.execute(query, vals)
            self.db.commit()
            cursor.close()
        except mysql.Error as err:
            print('++', err)
            print('ERROR mysql.Error: ', err.msg)
            # error = err.msg
            # cur.close()

    def dropColumn(self, column2Del):
        query = "ALTER TABLE {} DROP COLUMN %s;".format(self.tableName)
        query = "ALTER TABLE {} DROP COLUMN {};".format(self.tableName, column2Del)
        print(query)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def executeQuery(self, query):
        print(sys._getframe().f_code.co_name)
        # print(file)
        cursor = self.db.cursor()
        if isinstance(query, str):
            commands = query.split(';')
        for command in commands:
            print('executing command: ', command)
            cursor.execute(command)

    def importCSV(self, inpFile):
        with open(inpFile, 'r', encoding='unicode_escape', newline='') as csvfile:
            linereader = csv.reader(csvfile, delimiter=';', quotechar='|')
            for i, row in enumerate(linereader):
                if i == 0:
                    tableHead = row
                    continue
                if '' in row or 'None' in row:
                    new_strings = []
                    for string in row:
                        if string == '' or string == 'None':
                            new_strings.append(None)
                        else:
                            new_strings.append(string)
                    row = new_strings
                self.add_row(tableHead, row)

    def importSparkasseCSV(self, inpFile):
        with open(inpFile, 'r', encoding='unicode_escape', newline='') as csvfile:
            linereader = csv.reader(csvfile, delimiter=';', quotechar='"')
            indxBuchungstag = self.columnsNames.index('Buchungstag')
            indxValutDatum = self.columnsNames.index('Valutadatum')
            indxBetrag = self.columnsNames.index('Betrag')
            for i, row in enumerate(linereader):
                # print(row)
                row.insert(0, i)
                if i == 0:
                    tableHead = row
                    continue
                if '' in row or 'None' in row:
                    new_strings = []
                    for string in row:
                        if string == '' or string == 'None':
                            new_strings.append(None)
                        else:
                            new_strings.append(string)
                    row = new_strings
                buchungstag = row[indxBuchungstag]
                valutDatum = row[indxValutDatum]
                betrag = row[indxBetrag]
                betrag = float(betrag.replace(',', '.'))
                buchungstag = self.convertDatumFormat4SQL(buchungstag)
                valutDatum = self.convertDatumFormat4SQL(valutDatum)

                row[indxBuchungstag] = buchungstag
                row[indxValutDatum] = valutDatum
                row[indxBetrag] = betrag
                self.add_row(self.columnsNames, row)

    def convertDatumFormat4SQL(self, datum):
        # print(sys._getframe().f_code.co_name)
        # newDate = datetime.strptime(datum, '%d.%m.%y')
        for fmt in ('%Y-%m-%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
            try:
                newDate = datetime.strptime(datum, fmt)
                return newDate.date()
            except ValueError:
                pass
        raise ValueError('no valid date format found')

    def convertTimeFormat4SQL(self, time):
        # print(sys._getframe().f_code.co_name)
        # newDate = datetime.strptime(datum, '%d.%m.%y')
        for fmt in ('%H:%M', '%H:%M:%S'):
            try:
                newDate = datetime.strptime(time, fmt)
                return newDate.time()
            except ValueError:
                pass
        raise ValueError('no valid date format found')

    def returnColumn(self, col):
        query = 'SELECT {} FROM {}'.format(col, self.tableName)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i[0])
        return values

    def returnColumns(self, cols):
        strTableHead = ''
        for col in cols:
            strTableHead += '{}, '.format(col)
        strTableHead = strTableHead[:-2]

        query = 'SELECT {} FROM {}'.format(strTableHead, self.tableName)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def returnCellsWhere(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(col, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN {}".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        # print('*****records', records, type(records))
        cursor.close()
        values = []
        # colType = self.get_column_type(col)
        for i in records:
            # print('ßßßßßßß', colType)
            # print('ßßßßßßß', i[0])
            # if colType == 'longblob':
            #     values.append(i[0])
            #     # continue
            # # elif colType == 'json':
            # #     values.append(json.loads(i[0]))
            # else:
            values.append(i[0])
        return values

    def returnCellsWhereDiffrent(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} != '{}'".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} != "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        # print(records)
        cursor.close()
        values = []
        colType = self.get_column_type(col)
        for i in records:
            if colType == 'json':
                values.append(json.loads(i[0]))
            else:
                values.append(i[0])
        return values

    def returnColsWhere(self, cols, matches):
        relCols = ''
        for col in cols:
            relCols += '{}, '.format(col)
        relCols = relCols[:-2]

        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if searchKey is None:
                query = "SELECT {} FROM {} WHERE {} IS NULL".format(relCols, self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    new = '{} IS NULL AND '.format(searchCol)
                else:
                    new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(relCols, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def write_file(self, data, filename):
        # Convert binary data to proper format and write it on Hard Disk
        with open(filename, 'wb') as file:
            file.write(data)

    def show_create_table_2(self, tableName):
        print('################')
        query = 'SELECT * FROM {}'.format(self.tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        # print('++', query)
        data = ""
        for row in records:
            # print('ÄÄ', row)
            data += "INSERT INTO `" + str(self.tableName) + "` VALUES("
            first = True
            for field in row:
                if not first:
                    data += ', '
                data += '"' + str(field) + '"'
                first = False

            data += ");\n"
        data += "\n\n"

        values = []
        # for i in data:
        #     print(i)
        cursor.close()
        #     # values.append(i[0])
        # # return values
        # filename = str(os.getenv("HOME")) + "/backup_" + now.strftime("%Y-%m-%d_%H:%M") + ".sql"
        filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\bb.sql"
        # data = records[1]
        FILE = open(filename, "w")
        FILE.writelines(data)
        FILE.close()

    def compare_sql_file_to_sql_table(self, sql_file):
        f = open(sql_file)
        same = True
        diffrences = {}
        for line in f.read().splitlines():
            spl_row = line.split()
            if spl_row and '`' in spl_row[0]:
                sql_file_col = spl_row[0].strip('`')
                if sql_file_col in self.columnsDetProperties.keys():
                    col_type_file = spl_row[1]
                    col_type_sql = self.columnsDetProperties[sql_file_col][0]
                    if col_type_file != col_type_sql:
                        txt = 'Column {} has a different format in file than in sql table'.format(sql_file_col)
                        # print(txt)
                        same = False
                        diffrences[sql_file_col] = txt
                        # raise RuntimeError(txt)
                else:
                    raise RuntimeError('Column {} missing in sql table'.format(sql_file_col))
        f.close()
        if not same:
            return diffrences

if __name__ == '__main__':

    # iniFile = r"D:\Python\MySQL\database.ini"
    # iniFile = r"D:\Python\MySQL\web_db.ini"
    # iniFile = r"D:\Python\MySQL\mysqlquerys\src\mysqlquerys\local.ini"
    # iniFile = r"C:\_Development\Diverse\pypi\cfgmRN.ini"
    iniFile = r"C:\_Development\Diverse\pypi\cfgm.ini"

    conf = connect.Config(iniFile)
    # print(conf.credentials)

    db = DataBase(conf.credentials)
    print(db.is_connected)
    # tables = ['users', 'all_cars', 'hyundai_ioniq', 'bmw_318i']
    # sql = r'C:\_Development\Diverse\pypi\radu\masina\src\masina\ee.sql'
    # tables_dict = {'users': 'id',
    #       'all_cars': 'user_id',
    #       'hyundai_ioniq': 'id_users',
    #       'bmw_318i': 'id_users',
    #       }
    # print(db.backup_profile_with_id(tables_dict, sql, user_id=1))
    # print(table.returnAllRecordsFromTable())
    testrrrr = Table(conf.credentials, 'users')
    # print(testrrrr.children_tables)
    # print(testrrrr.columnsDetProperties)
    sql_file = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\static\sql\users_template.sql"
    testrrrr.compare_sql_file_to_sql_table(sql_file)




import numpy as np
import os
from PyQt5.QtWidgets import *
from PyQt5 import uic
import pathlib
import sys
import subprocess
import rappmysql
from rappmysql.mruser.myusers import Users
from rappmysql.mysqlquerys import connect
np.set_printoptions(linewidth=600)

compName = os.getenv('COMPUTERNAME')


class CheckRequiredFiles:
    def __init__(self):
        path2GUI = pathlib.Path(__file__)
        self.path2GUI = path2GUI.resolve(path2GUI).parent / 'GUI'
        self.gui = self.path2GUI / 'login_window.ui'
        # self.postgresql_gui = self.path2GUI / 'postgresql_gui.ui'
        # self.mysql_gui = self.path2GUI / 'postgresql_gui.ui'
        # self.filterWindow = self.path2GUI / 'filterWindow.ui'
        # self.inputsWindow = self.path2GUI / 'inputs_sql.ui'


# class guiSqlRM(QMainWindow, CheckRequiredFiles):
#     def __init__(self, connection):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         super(guiSqlRM, self).__init__()
#         Ui_MysqlWindow, QtBaseClass = uic.loadUiType(self.mysql_gui)
#         self.ui = Ui_MysqlWindow()
#         self.ui.setupUi(self)
#         self.credentials = connection.credentials
#         self.db_type = connection.db_type
#         if self.db_type == 'mysql':
#             self.sql_rm = mysql_rm
#             self.ui.TBox_db_sch_tab.setItemEnabled(1, False)
#         elif self.db_type == 'postgresql':
#             self.sql_rm = postgresql_rm
#
#         self.ui.GB_db_defs.setVisible(False)
#         self.ui.GB_schema_defs.setVisible(False)
#         self.ui.LW_schemas_available_in_db.setVisible(False)
#         self.ui.GB_table_defs.setVisible(False)
#
#         self.GB_db_defs = self.ui.GB_db_defs
#         self.GB_db_not_callable = self.ui.GB_db_not_callable
#         self.GB_db_callable_without_args = self.ui.GB_db_callable_without_args
#         self.GB_db_callable_with_args = self.ui.GB_db_callable_with_args
#         self.GB_schema_not_callable = self.ui.GB_schema_not_callable
#         self.GB_schema_callable_without_args = self.ui.GB_schema_callable_without_args
#         self.GB_schema_callable_with_args = self.ui.GB_schema_callable_with_args
#         self.GB_table_defs = self.ui.GB_table_defs
#         self.GB_table_not_callable = self.ui.GB_table_not_callable
#         self.GB_table_callable_without_args = self.ui.GB_table_callable_without_args
#         self.GB_table_callable_with_args = self.ui.GB_table_callable_with_args
#         self.ui.TBox_db_sch_tab.setCurrentIndex(0)
#
#         self.ui.CheckB_db_methods.toggled.connect(self.populate_class_methods)
#         self.ui.CheckB_schema_methods.toggled.connect(self.populate_class_methods)
#         self.ui.CheckB_table_methods.toggled.connect(self.populate_class_methods)
#
#         self.db = self.sql_rm.DataBase(self.credentials)
#         title = '{} {}'.format(self.credentials['database'], self.db.dataBaseVersion)
#         self.setWindowTitle(title)
#
#         self.ui.CheckB_hide_schemas_list.toggled.connect(lambda: self.populate_list(self.db.allAvailableTablesInDatabase, self.ui.LW_schemas_available_in_db))
#         self.ui.CheckB_hide_tables_list.toggled.connect(lambda: self.populate_list(self.schema.allAvailableTablesInSchema, self.ui.LW_tables_available))
#         self.ui.CheckB_hide_table.toggled.connect(lambda: self.hide_table_widget(self.ui.TW_table_data))
#
#         if self.db_type == 'mysql':
#             self.populate_list(self.db.allAvailableTablesInDatabase, self.ui.LW_schemas_available_in_db)
#             self.ui.LW_schemas_available_in_db.itemDoubleClicked.connect(self.connect_to_table)
#         elif self.db_type == 'postgresql':
#             self.populate_list(self.db.allAvailableSchemas, self.ui.LW_schemas_available_in_db)
#             self.ui.LW_schemas_available_in_db.itemDoubleClicked.connect(self.connect_to_schema)
#             self.ui.LW_tables_available.itemDoubleClicked.connect(self.connect_to_table)
#
#         self.ui.TW_table_data.horizontalHeader().sectionClicked.connect(self.sort)
#         self.ui.TW_table_data.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
#         self.ui.PB_default_filter.clicked.connect(self.set_default_filter_filter)
#         self.ui.PB_show_full_table.clicked.connect(self.show_full_table)
#
#     def populate_list(self, list_items, list_widget):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if isinstance(self.sender(), QCheckBox):
#             if self.sender().isChecked():
#                 list_widget.setVisible(False)
#                 list_widget.clear()
#                 for i, schema in enumerate(list_items):
#                     list_widget.insertItem(i, schema)
#             else:
#                 list_widget.setVisible(True)
#         elif isinstance(self.sender(), QPushButton):
#             list_widget.setVisible(True)
#             list_widget.clear()
#             for i, schema in enumerate(list_items):
#                 list_widget.insertItem(i, schema)
#
#     def hide_table_widget(self, table_widget):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if isinstance(self.sender(), QCheckBox):
#             if self.sender().isChecked():
#                 table_widget.setVisible(False)
#             else:
#                 table_widget.setVisible(True)
#
#     def populate_class_methods(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if 'DataBase' in self.sender().text():
#             module_class = self.sql_rm.DataBase
#             module_instance = self.db
#             module_class_name = 'self.sql_rm.DataBase'
#             group_box = self.GB_db_defs
#             GB_not_callable = self.GB_db_not_callable
#             GB_callable_without_args = self.GB_db_callable_without_args
#             GB_callable_with_args = self.GB_db_callable_with_args
#             definitions = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
#         elif 'Schema' in self.sender().text():
#             module_class = self.sql_rm.Schema
#             module_instance = self.schema
#             module_class_name = 'self.sql_rm.Schema'
#             group_box = self.GB_schema_defs
#             GB_not_callable = self.GB_schema_not_callable
#             GB_callable_without_args = self.GB_schema_callable_without_args
#             GB_callable_with_args = self.GB_schema_callable_with_args
#             definitions_DB = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
#             definitions = [x for x in dir(self.sql_rm.Schema) if not x.startswith('__')]
#             for item in definitions:
#                 if item in definitions_DB:
#                     definitions.remove(item)
#         elif 'Table' in self.sender().text():
#             module_class = self.sql_rm.Table
#             module_instance = self.table
#             module_class_name = 'self.sql_rm.Table'
#             group_box = self.GB_table_defs
#             GB_not_callable = self.GB_table_not_callable
#             GB_callable_without_args = self.GB_table_callable_without_args
#             GB_callable_with_args = self.GB_table_callable_with_args
#             definitions_DB = [x for x in dir(self.sql_rm.DataBase) if not x.startswith('__')]
#             definitions = [x for x in dir(self.sql_rm.Table) if not x.startswith('__')]
#             new_def = []
#             for item in definitions:
#                 if item in definitions_DB:
#                     continue
#                 else:
#                     new_def.append(item)
#             definitions = new_def
#
#         if isinstance(self.sender(), QCheckBox):
#             if self.sender().isChecked():
#                 group_box.setVisible(True)
#                 vbox_not_callable = QGridLayout()
#                 vbox_callable_without_args = QGridLayout()
#                 vbox_callable_with_args = QGridLayout()
#                 for i, meth_name in enumerate(definitions):
#                     radio = QRadioButton(meth_name)
#                     radio.col = 0
#                     radio.row = i
#                     radio.module_class = module_class
#                     radio.module_instance = module_instance
#                     radio.meth_name = meth_name
#                     meth = getattr(module_class, meth_name)
#                     if callable(meth):
#                        #print('meth', meth_name)
#                         docs = None
#                         # if hasattr(meth, '__doc__'):
#                         #     docs = getattr(meth, '__doc__')
#                         varname = 'def_args'
#                         loc = locals()
#                         exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth_name), globals(), loc)
#                         meth_args = (loc[varname]).args
#                         coloana = 1
#                         if len(meth_args) == 1 and meth_args[0] == 'self':
#                             radio.toggled.connect(self.start_noncallable_method)
#                             vbox_callable_without_args.addWidget(radio, i, radio.col)
#                         else:
#                             vbox_callable_with_args.addWidget(radio, i, radio.col)
#                             tool_button = QPushButton('inputs')
#                             tool_button.meth_name = meth_name
#                             tool_button.clicked.connect(self.get_method_arguments)
#                             vbox_callable_with_args.addWidget(tool_button, i, 2)
#                             push_button = QPushButton('Fire')
#                             push_button.row = len(definitions) + 1
#                             push_button.col = 2
#                             push_button.clicked.connect(self.start_callable_method)
#                             vbox_callable_with_args.addWidget(push_button, push_button.row, push_button.col)
#                     else:
#                         radio.toggled.connect(self.start_noncallable_method)
#                         vbox_not_callable.addWidget(radio, i, radio.col)
#                 GB_not_callable.setLayout(vbox_not_callable)
#                 GB_callable_without_args.setLayout(vbox_callable_without_args)
#                 GB_callable_with_args.setLayout(vbox_callable_with_args)
#             else:
#                 group_box.setVisible(False)
#
#     def get_method_arguments(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if self.ui.TBox_db_sch_tab.currentIndex() == 0:
#             layout_grupa = self.GB_callable_with_args.layout()
#             module_class_name = 'self.sql_rm.DataBase'
#             module_instance = self.db
#         elif self.ui.TBox_db_sch_tab.currentIndex() == 1:
#             layout_grupa = self.GB_schema_callable_with_args.layout()
#             module_class_name = 'self.sql_rm.DataBase'
#             module_instance = self.db
#         elif self.ui.TBox_db_sch_tab.currentIndex() == 2:
#             layout_grupa = self.GB_table_callable_with_args.layout()
#             module_class_name = 'self.sql_rm.Table'
#             module_instance = self.table
#         args_dict = {}
#         meth = None
#         for x in range(layout_grupa.count()):
#             item = layout_grupa.itemAt(x)
#             widget = item.widget()
#             if isinstance(widget, QRadioButton):
#                 if widget.isChecked():
#                     meth = widget.text()
#                    #print(meth)
#                     varname = 'def_args'
#                     loc = locals()
#                     exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth), globals(), loc)
#                     meth_args = (loc[varname]).args
#                    #print('\t', meth_args)
#
#                     docs = None
#                     if hasattr(meth, '__doc__'):
#                         docs = getattr(meth, '__doc__')
#                     for co, arg in enumerate(meth_args):
#                         if arg == 'self':
#                             continue
#
#                         if docs:
#                             for line in docs.split('\n'):
#                                 line = line.strip()
#                                #print(line)
#                                 if ':param' in line and arg in line:
#                                     if 'QFileDialog.getOpenFileNames' in line:
#                                        #print('BINGO')
#
#     def connect_to_schema(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         schema_name = self.ui.LW_schemas_available_in_db.currentItem().text()
#         self.schema = self.sql_rm.Schema(self.credentials, schema_name)
#         self.schema.set_schema_as_default()
#         self.ui.TBox_db_sch_tab.setCurrentIndex(1)
#         list_items = self.schema.allAvailableTablesInSchema
#         list_widget = self.ui.LW_tables_available
#         list_widget.setVisible(True)
#         list_widget.clear()
#         for i, schema in enumerate(list_items):
#             list_widget.insertItem(i, schema)
#
#     def connect_to_table(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         self.filterList = []
#         if self.db_type == 'mysql':
#             table_name = self.ui.LW_schemas_available_in_db.currentItem().text()
#             self.table = self.sql_rm.Table(self.credentials, table_name)
#         elif self.db_type == 'postgresql':
#             schema_name = self.ui.LW_schemas_available_in_db.currentItem().text()
#             table_name = self.ui.LW_tables_available.currentItem().text()
#             self.table = self.sql_rm.Table(self.credentials, schema_name, table_name)
#
#         self.ui.LE_tot_no_of_rows.setText(str(self.table.noOfRows))
#         self.ui.TBox_db_sch_tab.setCurrentIndex(2)
#         table_data = self.table.returnLastRecords('id', 100)
#         table_data = np.atleast_2d(table_data)
#         self.ui.lineEditFilterList.setText("last 100 id's")
#         self.populate_table(table_data)
#
#     def populate_table(self, table_data):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         self.ui.TW_table_data.clear()
#         self.ui.TW_table_data.setRowCount(table_data.shape[0])
#         self.ui.TW_table_data.setColumnCount(table_data.shape[1])
#         tableHead = self.table.columnsNames
#         self.ui.TW_table_data.setHorizontalHeaderLabels(tableHead)
#         for col in range(table_data.shape[1]):
#             for row in range(table_data.shape[0]):
#                 if isinstance(table_data[row, col], int):
#                     item = QTableWidgetItem()
#                     item.setData(QtCore.Qt.DisplayRole, table_data[row, col])
#                 else:
#                     item = QTableWidgetItem(str(table_data[row, col]))
#                 self.ui.TW_table_data.setItem(row, col, item)
#         self.ui.LE_showing.setText(str(table_data.shape[0]))
#
#     def deleteItemsOfLayout(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         layout_grupa = self.GB_db_defs.layout()
#         if layout_grupa is not None:
#             while layout_grupa.count():
#                 item = layout_grupa.takeAt(0)
#                 widget = item.widget()
#                 if widget is not None:
#                     widget.setParent(None)
#                 else:
#                     self.deleteItemsOfLayout(item.layout())
#             sip.delete(layout_grupa)
#
#     def start_noncallable_method(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if self.sender().isChecked():
#             method = self.sender().text()
#             if not callable(getattr(self.sender().module_class, method)):
#                 text = getattr(self.sender().module_instance, self.sender().text())
#                 self.ui.TE_result.setPlainText(str(text))
#             else:
#                 text = getattr(self.sender().module_instance, self.sender().text())()
#                 self.ui.TE_result.setPlainText(str(text))
#
#     def start_callable_method(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if self.ui.TBox_db_sch_tab.currentIndex() == 0:
#             layout_grupa = self.GB_db_defs.layout()
#             module_class_name = 'self.sql_rm.DataBase'
#             module_instance = self.db
#         elif self.ui.TBox_db_sch_tab.currentIndex() == 1:
#             layout_grupa = self.GB_schema_defs.layout()
#             module_class_name = 'self.sql_rm.Schema'
#             module_instance = self.schema
#         elif self.ui.TBox_db_sch_tab.currentIndex() == 2:
#             layout_grupa = self.GB_table_defs.layout()
#             module_class_name = 'self.sql_rm.Table'
#             module_instance = self.table
#
#         args_dict = {}
#         meth = None
#         for x in range(layout_grupa.count()):
#             item = layout_grupa.itemAt(x)
#             widget = item.widget()
#             if isinstance(widget, QRadioButton):
#                 if widget.isChecked():
#                     meth = widget.text()
#                    #print(meth)
#                     varname = 'def_args'
#                     loc = locals()
#                     exec('{}=inspect.getfullargspec({}.{})'.format(varname, module_class_name, meth), globals(), loc)
#                     meth_args = (loc[varname]).args
#                     for arg in meth_args:
#                         if arg == 'self':
#                             continue
#                         for x in range(layout_grupa.count()):
#                             item = layout_grupa.itemAt(x)
#                             widget = item.widget()
#                             if isinstance(widget, QLineEdit):
#                                 if widget.arument == arg:
#                                     args_dict[arg] = widget.text()
#                                     break
#                             elif isinstance(widget, QListWidget):
#                                 if widget.arument == arg:
#                                     args_dict = []
#                                     for i in range(widget.count()):
#                                         args_dict.append(widget.item(i).text())
#                                         break
#         if not meth:
#             message = "Please select the a method"
#             QMessageBox.warning(self, 'Fehlende Daten', message, QMessageBox.Ok)
#             return
#
#         text = getattr(module_instance, meth)(**args_dict)
#         self.ui.TE_result.setPlainText(str(text))
#
#     def sort(self, logical_index):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         header = self.ui.TW_table_data.horizontalHeader()
#         order = Qt.DescendingOrder
#         if not header.isSortIndicatorShown():
#             header.setSortIndicatorShown(True)
#         elif header.sortIndicatorSection() == logical_index:
#             order = header.sortIndicatorOrder()
#         header.setSortIndicator(logical_index, order)
#         self.ui.TW_table_data.sortItems(logical_index, order)
#
#     def setFilter(self, logical_index):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         colName = self.table.columnsNames[logical_index]
#
#         if self.table.columnsProperties[colName] == 'datetime' or \
#                 self.table.columnsProperties[colName] == 'timestamp' or \
#                 self.table.columnsProperties[colName] == 'timestamp without time zone':
#             filt = FilterWindow.getDateTimeInterval(colName)
#             if not filt:
#                 return
#             if isinstance(filt, tuple):
#                 minDate, maxDate = filt
#                 minDate = minDate.toPyDateTime()
#                 maxDate = maxDate.toPyDateTime()
#                 filterVals = (str(minDate), str(maxDate))
#             elif isinstance(filt, QDateTime):
#                 filterVals = filt.toPyDateTime()
#             self.applyFilter(colName, filterVals)
#         elif self.table.columnsProperties[colName] == 'time' or \
#                 self.table.columnsProperties[colName] == 'time without time zone':
#             filt = FilterWindow.getTimeInterval(colName)
#             if not filt:
#                 return
#             if isinstance(filt, tuple):
#                 minTime, maxTime = filt
#                 minTime = minTime.toPyTime()
#                 maxTime = maxTime.toPyTime()
#                 filterVals = (str(minTime), str(maxTime))
#             elif isinstance(filt, QTime):
#                 filterVals = filt.toPyTime()
#             self.applyFilter(colName, filterVals)
#         elif self.table.columnsProperties[colName] == 'int' or \
#                 self.table.columnsProperties[colName] == 'integer' or \
#                 self.table.columnsProperties[colName] == 'mediumint':
#             filt = FilterWindow.getIntInterval(colName)
#             if not filt:
#                 return
#             if isinstance(filt, tuple):
#                 minTime, maxTime = filt
#                 filterVals = (str(minTime), str(maxTime))
#             elif isinstance(filt, str):
#                 filterVals = filt
#             self.applyFilter(colName, filterVals)
#         else:
#             header = self.ui.TW_table_data.horizontalHeader()
#             geom = QtCore.QRect(header.sectionViewportPosition(logical_index), 0, header.sectionSize(logical_index),
#                                 header.height())
#             item = QLineEdit(header)
#             item.setGeometry(geom)
#             item.show()
#             item.setFocus()
#             item.editingFinished.connect(lambda: (self.applyFilter(colName, item.text()), item.clear(),
#                                                   item.hide(), item.deleteLater()))
#
#     def applyFilter(self, colName, filter):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         if filter == "":
#             return
#
#         filterText = self.ui.lineEditFilterList.text()
#         if filterText == "last 100 id's":
#             filterText = ""
#             self.ui.lineEditFilterList.setText(filterText)
#         if not filterText:
#             if isinstance(filter, str):
#                 filterText += '{}="{}"'.format(colName, filter)
#             elif isinstance(filter, tuple):
#                 filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
#             elif isinstance(filter, list):
#                 filterText += '{} in {}'.format(str(filter), colName)
#         else:
#             if isinstance(filter, str):
#                 filterText += '; {}="{}"'.format(colName, filter)
#             elif isinstance(filter, tuple):
#                 filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
#             elif isinstance(filter, list):
#                 filterText += '{} in {}'.format(str(filter), colName)
#
#         self.ui.lineEditFilterList.setText(filterText)
#         tup = (colName, filter)
#         self.filterList.append(tup)
#
#         table_data = self.table.filterRows(self.filterList)
#
#         if table_data:
#             table_data = np.atleast_2d(table_data)
#         else:
#             table_data = np.zeros((0, len(self.tableHead)), dtype=str)
#
#         self.populate_table(table_data)
#
#     def set_default_filter_filter(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         table_data = self.table.returnLastRecords('id', 100)
#         table_data = np.atleast_2d(table_data)
#         self.populate_table(table_data)
#         self.ui.lineEditFilterList.setText("last 100 id's")
#
#     def show_full_table(self):
#        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
#         table_data = self.table.returnAllRecordsFromTable()
#         table_data = np.atleast_2d(table_data)
#         self.populate_table(table_data)
#         self.ui.lineEditFilterList.setText("")
#
#
# class FilterWindow(QDialog, CheckRequiredFiles):
#     def __init__(self, colType, colName):
#         super(FilterWindow, self).__init__()
#         self.colName = colName
#         Ui_MainWindow, QtBaseClass = uic.loadUiType(self.filterWindow)
#         self.gui = Ui_MainWindow()
#         self.gui.setupUi(self)
#         self.gui.GBDateTime.setVisible(False)
#         self.gui.GBLineEdit.setVisible(False)
#         self.gui.GBTime.setVisible(False)
#         self.gui.checkBoxDateTime.stateChanged.connect(self.openDateTimeInterval)
#         self.gui.checkBoxTime.stateChanged.connect(self.openTimeInterval)
#         self.gui.checkBoxMaxVal.stateChanged.connect(self.openValInterval)
#         if colType == 'dateTime':
#             self.gui.GBDateTime.setVisible(True)
#             self.gui.dateTimeEdit_MaxVal.setEnabled(False)
#             self.gui.label_dateTime.setText('= {}'.format(self.colName))
#             self.gui.dateTimeEdit_MinVal.setDateTime(QDateTime(datetime.now().year,
#                                                                datetime.now().month,
#                                                                1, 0, 0, 0))
#             self.gui.dateTimeEdit_MaxVal.setDateTime(QDateTime(datetime.now().year,
#                                                                datetime.now().month,
#                                                                datetime.now().day,
#                                                                datetime.now().hour,
#                                                                datetime.now().minute,
#                                                                datetime.now().second))
#         elif colType == 'time':
#             self.gui.GBTime.setVisible(True)
#             self.gui.timeEditMax.setEnabled(False)
#         elif colType == 'int':
#             self.gui.GBLineEdit.setVisible(True)
#             self.gui.lineEditMax.setEnabled(False)
#
#         self.rejected.connect(self.byebye)
#
#     def openDateTimeInterval(self):
#         if self.gui.checkBoxDateTime.isChecked():
#             self.gui.dateTimeEdit_MaxVal.setEnabled(True)
#             self.gui.label_dateTime.setText('< {} <'.format(self.colName))
#         else:
#             self.gui.dateTimeEdit_MaxVal.setEnabled(False)
#             self.gui.label_dateTime.setText('= {}'.format(self.colName))
#
#     def openTimeInterval(self):
#         if self.gui.checkBoxTime.isChecked():
#             self.gui.timeEditMax.setEnabled(True)
#             self.gui.label_Time.setText('< {} <'.format(self.colName))
#         else:
#             self.gui.timeEditMax.setEnabled(False)
#             self.gui.label_Time.setText('= {}'.format(self.colName))
#
#     def openValInterval(self):
#         if self.gui.checkBoxMaxVal.isChecked():
#             self.gui.lineEditMax.setEnabled(True)
#             self.gui.label_val.setText('< {} <'.format(self.colName))
#         else:
#             self.gui.lineEditMax.setEnabled(False)
#             self.gui.label_val.setText('= {}'.format(self.colName))
#
#     def byebye(self):
#         self.close()
#
#     def dateTimeInterval(self):
#         if self.gui.checkBoxDateTime.isChecked():
#             tup = (self.gui.dateTimeEdit_MinVal.dateTime(), self.gui.dateTimeEdit_MaxVal.dateTime())
#             return tup
#         else:
#             return self.gui.dateTimeEdit_MinVal.dateTime()
#
#     def timeInterval(self):
#         if self.gui.checkBoxTime.isChecked():
#             tup = (self.gui.timeEditMin.time(), self.gui.timeEditMax.time())
#             return tup
#         else:
#             return self.gui.timeEditMin.time()
#
#     def intInterval(self):
#         if self.gui.checkBoxMaxVal.isChecked():
#             tup = (self.gui.lineEditMin.text(), self.gui.lineEditMax.text())
#             return tup
#         else:
#             return self.gui.lineEditMin.text()
#
#     @staticmethod
#     def getDateTimeInterval(colName):
#         dialog = FilterWindow('dateTime', colName)
#         result = dialog.exec_()
#         filt = dialog.dateTimeInterval()
#         if result == QDialog.Accepted:
#             return filt
#         else:
#             return None
#
#     @staticmethod
#     def getTimeInterval(colName):
#         dialog = FilterWindow('time', colName)
#         result = dialog.exec_()
#         filt = dialog.timeInterval()
#         if result == QDialog.Accepted:
#             return filt
#         else:
#             return None
#
#     @staticmethod
#     def getIntInterval(colName):
#         dialog = FilterWindow('int', colName)
#         result = dialog.exec_()
#         filt = dialog.intInterval()
#         if result == QDialog.Accepted:
#             return filt
#         else:
#             return None
#
#
# class InputsSQL(QDialog, CheckRequiredFiles):
#     def __init__(self, colType, colName):
#         super(InputsSQL, self).__init__()
#         Ui_MainWindow, QtBaseClass = uic.loadUiType(self.inputsWindow)
#         self.gui = Ui_MainWindow()
#         self.gui.setupUi(self)


class LogInWindow(QMainWindow, CheckRequiredFiles):
    def __init__(self):
        # print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
        #                                                           sys._getframe().f_back.f_code.co_name))
        super(LogInWindow, self).__init__()
        Ui_MainWindow, QtBaseClass = uic.loadUiType(self.gui)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.conf = connect.Config(rappmysql.ini_users)
        self.setWindowTitle('{}'.format(self.conf.credentials['database']))

        if compName == 'DESKTOP-5HHINGF' or compName == 'MPCC6995':
                self.ui.LE_user_name.setText('radu')
                self.ui.LE_password.setText('9876')
                # self.login_user()
                # zipFile = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\2024_12_13__16_44_16_000000001.zip"
                # self.app_users.import_profile_with_files(zipFile, import_files=False)
                # self.login_user()


    #     self.conf = None
    #     self.GB_dbs_available = self.ui.GB_dbs_available
        self.ui.GB_register_new_user.setVisible(False)
    #     self.ui.GB_dbs_available.setVisible(False)
    #     self.ui.TB_load_config_ini.clicked.connect(lambda: (self.load_config_ini(), self.read_config_ini()))
        self.ui.PB_Login.clicked.connect(self.login_user)
        self.ui.PB_registe_new_user.clicked.connect(self.register_new_user)
        self.ui.PB_erase_all_traces.clicked.connect(self.erase_all_traces)
        self.ui.PB_complete_backup.clicked.connect(self.complete_backup)
        self.ui.PB_complete_profile_backup.clicked.connect(self.complete_backup)
        self.ui.LW_user_applications.itemDoubleClicked.connect(self.start_app)
        self.ui.PB_export_profile.clicked.connect(self.export_full_profile)
        self.ui.PB_import_profile.clicked.connect(self.import_profile)

    def read_credentials(self):
       #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        user_name = self.ui.LE_user_name.text()
        password = self.ui.LE_password.text()
        return user_name, password

    def login_user(self):
        self.user_name, password = self.read_credentials()
        self.user = Users(self.user_name, rappmysql.ini_users)
        self.list_user_applications(self.user.applications, self.user.unused_applications)
        return self.user_name, password

    def register_new_user(self):
        self.ui.GB_register_new_user.setVisible(True)

    def list_user_applications(self, applications, unused_applications):
        for i, schema in enumerate(applications):
            self.ui.LW_user_applications.insertItem(i, schema)
        for i, schema in enumerate(unused_applications):
            self.ui.LW_not_used_applications.insertItem(i, schema)

    def start_app(self):
        app_name = self.ui.LW_user_applications.currentItem().text()
        ##print(app_name)
        if app_name == 'cheltuieli':
            cmd = ['python', '-m', 'rappmysql.cheltuieli.chelt_gui_desktop']
            print('+++++', cmd)
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 universal_newlines=True)
            out, err = p.communicate()

            if p.returncode != 0:
               print('returncode: ', p.returncode)
               print('ERROR', err)
            return p.returncode
        elif app_name == 'masina':
            cmd = ['python', '-m', 'rappmysql.masina.auto_gui_desktop']
           #print('+++++', cmd)
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 universal_newlines=True)
            out, err = p.communicate()

            if p.returncode != 0:
               print('returncode: ', p.returncode)
               print('ERROR', err)
            return p.returncode
        elif app_name == 'aeroclub':
            cmd = ['python', '-m', 'rappmysql.aeroclub.aeroclub_gui']
           #print('+++++', cmd)
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 universal_newlines=True)
            out, err = p.communicate()

            if p.returncode != 0:
               print('returncode: ', p.returncode)
               print('ERROR', err)
            return p.returncode

    def erase_all_traces(self):
        user = Users(None, rappmysql.ini_users)
        user.erase_all_traces()

    def complete_backup(self):
        if self.sender().text() == 'Complete_backup':
            self.user.backup(complete_database=True)
        if self.sender().text() == 'Complete_profile_backup':
            self.user.backup(complete_database=False)

    def export_full_profile(self):
        # expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "File (*.zip)")
        # expDir = QFileDialog.getExistingDirectory()
        # print(expDir)
        # if expName:
        if self.ui.CB_export_profile_with_files.isChecked():
            self.user.backup_users_tables(export_files=True)
        else:
            self.user.backup_users_tables(export_files=False)

    def import_profile(self):
        import_files=self.ui.CB_import_profile_with_files.isChecked()
        zipFile, _ = QFileDialog.getOpenFileName(self, "Open file", "", "File (*.zip)")
        if zipFile:
            user = Users(None, rappmysql.ini_users)
            user.import_zip(zipFile, import_files)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = LogInWindow()
    window.show()
    # sys.exit(app.exec_())
    app.exec_()

    # credentials = LogInWindow.get_login_credentials()
    # if credentials:
    #     user_name, password = credentials
    #    #print(user_name, password)

